macro FRETratio{

#@ File(label="Select input directory", style="directory") input_dir
#@ File(label="Select output directory", style="directory") output_dir
#@ File(label="Select background image", style="file",required="false") noisy_image
#@ Integer(label = "Donnor channel", value = 1) donnor_channel
#@ Integer(label = "Fret channel", value = 2) fret_channel
#@ String(label = "Time lapse", choices={"2D", "3D"}, style="radioButtonHorizontal") time_lapse
#@ Boolean(label = "Background image", value = true,  style="checkbox") bkg_image
#@ Boolean(label = "Manual thresholding", value = true, style="checkbox") manual_thresh
#@ Boolean(label = "Manual thresholding for each frame", value = true, style="checkbox") manual_thresh_frame

/* FRET ratiometric analysis of Schwann cells 
 * v0.5.3
*
* Read all Tiff files from input folder (nFxnCxnT); correct illumination; subtract background from noisy image;
* segment structure/cell and divide FRET/Donnor. Save output results with mean grey values along time for each position.  
* 
* This macro should NOT be redistributed without author's permission. 
* Explicit acknowledgement to the ALM facility should be done in case of published articles (approved in C.E. 7/17/2017):     
* 
* "The authors acknowledge the support of i3S Scientific Platform Advanced Light Microscopy, 
* member of the national infrastructure PPBI-Portuguese Platform of BioImaging (supported by POCI-01-0145-FEDER-022122)."
* 
* Date: May/2019
* Author: Mafalda Sousa, mafsousa@ibmc.up.pt 
* Advanced Ligth Microscopy, I3S 
* PPBI-Portuguese Platform of BioImaging
*/


setBatchMode(false);
count = 0;
n = 0;
countFiles(input_dir);
processFiles(input_dir, output_dir);
merge_excel_files();

//count the number of valid files in the directory     
function countFiles(dir) {
      list = getFileList(dir);
      for (i=0; i<list.length; i++) {
          if (endsWith(list[i], File.separator))
              countFiles("" + dir + list[i]);
          else
              count++;
      }
  }

// iterate through all the files in the folder in order to process them
function processFiles(input_dir, out_dir) {
      list = getFileList(input_dir);
      for (i=0; i<list.length; i++) {
          if (endsWith(list[i], File.separator))
              processFiles(""+input_dir+list[i], out_dir);
          else {
             showProgress(n++, count);
             path = input_dir + File.separator + list[i];
             file_name = list[i];
             if ( (endsWith(path, ".tif") || endsWith(path, ".tiff")) && File.exists(path)){
             	processFile(path,file_name);
             }
          }
      }
  }

//process each file and switch case depending on the input parameters 
function processFile(path, file_name) {
	   
		//open noisy image
	    if (bkg_image==true && File.exists(noisy_image) && endsWith(noisy_image, ".tif")){
	    
			open(noisy_image);
			N = getTitle();
	    }
	    
	    else if (bkg_image==true  ) {
	    	showMessage("warning","No background image found");
	    	break;     	
	    }
	    
		//open original image
		open(path);		
        T = getTitle();
		dotIndex = lastIndexOf(T, ".");
        title = substring(T, 0, dotIndex);                        
    
		run("Set Measurements...", "area mean min integrated display redirect=None decimal=3");
		
		if (time_lapse == "2D"  && bkg_image == true){			//CASE 2D with backgound image	
		 	FRET_ratio_noisyImage(T,N);
		}
		else if (time_lapse == "2D" && bkg_image == false){		//CASE 2D with no backgound image
			FRET_ratio(T);
		}
		else if (time_lapse == "3D" && bkg_image == true ){		//CASE time-lapse with backgound image
			TimeLapse_FRET_ratio_noisyImage(T,N);
		}
        else if (time_lapse == "3D" && bkg_image == false){		//CASE time-lapse with no backgound image
        	TimeLapse_FRET_ratio(T);
        }

}

//Fret ratio analysis for one time frame with noise image to remove background
function FRET_ratio_noisyImage(original,noise){			
		
		//shading correction of noise image
		selectWindow(noise);	
	    run("Polynomial Shading Corrector", "degree_x=1 degree_y=1 regularization=10");
	    wait(200);
 		//split channels 
		run("Split Channels"); 
		
		selectWindow("C" + donnor_channel + "-"+noise);
		donnor_noise = getTitle();
		selectWindow("C" + fret_channel + "-"+noise);
		fret_noise = getTitle();	    
	   
	    //shading correction of original image
	    selectWindow(original);
	    run("Polynomial Shading Corrector", "degree_x=1 degree_y=1 regularization=10");
	    wait(200);
	    //split channels 	   
	    run("Split Channels"); 
    	selectWindow("C" + donnor_channel + "-" + original);
    	donnor = getTitle();
    	selectWindow("C" + fret_channel + "-" + original);
    	fret = getTitle(); 	    

	                	    
		//subtract background in both channels (division by noisy image)	
		imageCalculator("Divide create 32-bit", fret, fret_noise);
	    fret_nobkg = getTitle();
	    selectWindow(fret);
	    close();
	    selectWindow(fret_noise);
	    close();
	    imageCalculator("Divide create 32-bit", donnor, donnor_noise);
	    donnor_nobkg = getTitle();
	    selectWindow(donnor);
	    close();
	    selectWindow(donnor_noise);
	    close();	

    	//cell segmentation => ROI mask
    	selectWindow(fret_nobkg);
	    run("Duplicate...", "title=tomask");
	    run("Median...", "radius=2");
	    if (manual_thresh == true){
	    	waitForUser("Select Image -> Adjust -> Threshold.. and select the best algorithm. Click OK after applying to all stack");
	    }
	    else {
	    	setAutoThreshold("Li dark");
			setOption("BlackBackground", true);
			run("Convert to Mask", "method=Li background=Default black"); 	
	    }
	  	run("Analyze Particles...", "size=100-Infinity show=Masks include add");
		roiManager("Select", 0);
		run("Create Mask");
		mask = getTitle();
		// devide mask by 255 (0-1)
	    run("Divide...", "value=255");	
	    
	    // Divide FRET by DONNOR (with no background) all image
	    imageCalculator("Divide create 32-bit stack", fret_nobkg, donnor_nobkg);
	    rename("Ratio_" + title);
	    ratio = getTitle();	  	    
	   
		//select ROI adnd measure inside it
	    roiManager("Select", 0);
	    run("Measure");		
 
		// Multiply the result by donnor mask in order to get only information inside the cell 
	    imageCalculator("Multiply create 32-bit stack", mask, ratio); 	    

	    //change LUT and save
	    run("royal");
		roiManager("Select", 0);
	    getStatistics(area, mean, min, max, std, histogram);
	    run("Enhance Contrast", "saturated=0.35");
	   // setMinAndMax(mean-3*std, mean+3*std);
	    wait(200);
	    print(title);
	    saveAs("Tiff", output_dir + File.separator + title + "_Ratio.tif");
	    run("RGB Color");
	    save_results(output_dir);

	    run("Close All");
	    close("Roi Manager");
	   if(isOpen("Results")){
       		list1 = getList("window.titles"); 
     		for (l=0; l<list1.length; l++){ 
   	    		winame = list1[l]; 
     			selectWindow(winame); 
     			run("Close"); 
     		} 	    	
	    }

}

//Fret ratio analysis for one time frame with no background image
function FRET_ratio(original){

		//correct illumination
		selectWindow(original);
	    run("Polynomial Shading Corrector", "degree_x=1 degree_y=1 regularization=10");	    
	   	run("Split Channels"); 
    	selectWindow("C" + donnor_channel + "-" + original);
    	donnor = getTitle();
    	selectWindow("C" + fret_channel + "-" + original);
    	fret = getTitle(); 	 	    
	    
	    selectWindow(fret);
	    setTool("rectangle");
	    waitForUser("Please select background with rectangular tool. Click OK when done");
 	    getBoundingRect(xmin, ymin, width, height);
 	    getSelectionCoordinates(x, y);
        makeSelection(0, x, y);
	    getStatistics(area,mean_fret);
	    print("Substracted BKG value: ",mean_fret);
	            	    
		//subtract fret background 	0
		run("Select None");    
	    selectWindow(fret);
	    run("Subtract...", "value=" + mean_fret);	
	    fret_nobkg = getTitle();

		//subtract donnor background manually
	    selectWindow(donnor);
    	makeSelection(0, x, y);
	    getStatistics(area,mean_donnor);
	    print("Substracted BKG value: ",mean_donnor);
	    run("Select None");   
    	run("Subtract...", "value=" + mean_donnor);
    	donnor_nobkg = getTitle();	

    	//cell segmentation => ROI mask
    	selectWindow(fret_nobkg);
	    run("Duplicate...", "title=tomask");
	    run("Median...", "radius=2");
	    if (manual_thresh == true){
	    	waitForUser("Select Image -> Adjust -> Threshold.. and select the best algorithm. Click OK after applying to all stack");
	    }
	    else {
	    	setAutoThreshold("Li dark");
			setOption("BlackBackground", true);
			run("Convert to Mask", "method=Li background=Default black"); 	
	    }
	  	run("Analyze Particles...", "size=100-Infinity show=Masks include add");
		roiManager("Select", 0);
		run("Create Mask");
		mask = getTitle();
		// devide mask by 255 (0-1)
	    run("Divide...", "value=255");	
	    
	    // Divide FRET by DONNOR (with no background)
	    imageCalculator("Divide create 32-bit stack", fret_nobkg, donnor_nobkg);
	    rename("Ratio_" + title);
	    ratio = getTitle();
	    
	    selectWindow(ratio);
		//select ROI and measure inside it
	    roiManager("Select", 0);
	    run("Measure");		

	    // Multiply the result by donnor mask in order to get only information inside the cell 
	    imageCalculator("Multiply create 32-bit", mask, ratio);

	    //change LUT
	    run("royal");
	    getStatistics(area, mean, min, max, std, histogram);	   
	    //setMinAndMax(mean-3*std, mean+3*std);	  
	    run("Enhance Contrast", "saturated=0.35");  
	    saveAs("Tiff", output_dir + File.separator + title + "_Ratio.tif");
	    run("RGB Color");
	    save_results(output_dir);		

    	run("Close All");
        close("Roi Manager");
       	if(isOpen("Results")){
       		list1 = getList("window.titles"); 
     		for (l=0; l<list1.length; l++){ 
   	    		winame = list1[l]; 
     			selectWindow(winame); 
     			run("Close"); 
     		} 	    	
	    }
}

//////NOT TESTED
//Fret ratio analysis for time-lapse with noisy image
function TimeLapse_FRET_ratio_noisyImage(original,noise){

		//shading correction of noise image
		 selectWindow(noise);	
	   //split channels 
		run("Split Channels"); 
		selectWindow("C" + donnor_channel + "-" + noise);
		donnor_noise = getTitle();
		selectWindow("C" + fret_channel + "-" + noise);
		fret_noise = getTitle();
		
		//shading correction of original image
	    selectWindow(original);	    
	    //run("Polynomial Shading Corrector", "degree_x=1 degree_y=1 regularization=10");	    
	    run("Split Channels"); 
    	selectWindow("C" + donnor_channel + "-" + original);
    	donnor = getTitle();
    	selectWindow("C" + fret_channel + "-" + original);
    	fret = getTitle(); 	 
	    
	   //subtract background in both channels (division by noisy image)	
	   //This will do shade coorrection and remove camera artiffacts		
	    imageCalculator("Subtract create 32-bit stack", fret, fret_noise);
	    fret_sc = getTitle();
	    selectWindow(fret);
	    close();
	    selectWindow(fret_noise);
	    close();
	    imageCalculator("Subtract create 32-bit stack", donnor, donnor_noise);
	    donnor_sc = getTitle();
	    selectWindow(donnor);
	    close();
	    selectWindow(donnor_noise);
	    close();	

	    //Remove background from shade correted images

	    selectWindow(fret_sc);
	    wait(200);
	    setTool("rectangle");
	    waitForUser("Please select background with rectangular tool. Click OK when done");
 	    getBoundingRect(xmin, ymin, width, height);
 	    getSelectionCoordinates(x, y);
        makeSelection(0, x, y);
	    getStatistics(area,mean_fret);
	    print("Substracted BKG value: ",mean_fret);
	            	    
		//subtract fret background 	
		run("Select None");    
	    selectWindow(fret_sc);   
 	   
	    for (j=1; j<=nSlices; j++) { // loop through slices in image
			setSlice(j);
			makeSelection(0, x, y);
			getStatistics(area,mean_fret);
			run("Select None");
	    	print("Substracted BKG value: ",mean_fret);
			run("Subtract...", "value="+mean_fret);
			run("Restore Selection");                
		}
		run("Select None");
	    	
	    fret_nobkg = getTitle();

		//subtract donnor background 
	    selectWindow(donnor_sc);
    	for (j=1; j<=nSlices; j++) { // loop through slices in image
			setSlice(j);
			makeSelection(0, x, y);
			getStatistics(area,mean_donnor);
			run("Select None");
	    	print("Substracted BKG value: ",mean_donnor);
			run("Subtract...", "value="+mean_donnor);
			run("Restore Selection");                
		}
		run("Select None");
    	donnor_nobkg = getTitle();	
	   
	    
	    //cell segmentation => ROI mask
	    selectWindow(fret_nobkg);    
    	
	    run("Duplicate...", "duplicate title=tomask");
	    run("Gaussian Blur...", "sigma=1 stack");
	    resetMinAndMax();
		run("8-bit");
		if (manual_thresh_frame == true){
			for (t=1;t<=nSlices; t++){
				setSlice(t);
				setAutoThreshold("Minimum dark");			
				waitForUser("Select Image -> Adjust -> Threshold.. and select the best threshold. DON'T APPLY and Click OK");				
				run("Convert to Mask", "method=Minimum background=Dark only");		
				 mask1 = getTitle();		
			}			
		}
		else{
	    	selectWindow(fret_nobkg);    
	    	
		    run("Duplicate...", "duplicate title=tomask");
		    run("Polynomial Shading Corrector", "degree_x=1 degree_y=1 regularization=10");
		    run("Gaussian Blur...", "sigma=2 stack");
		   
		    if (manual_thresh == true){
		    	setAutoThreshold("Minimum dark");	
		    	waitForUser("Select Image -> Adjust -> Threshold.. and select the best algorithm. Click OK after applying to all stack");
		    	 mask1 = getTitle();
		    	getThreshold(lower_threshold, upper);
		    }
		    else {
		    	setAutoThreshold("Minimum dark");
				setOption("BlackBackground", true);
				getThreshold(lower_threshold, upper);
				run("Convert to Mask", "method=Li background=Default black"); 	
				mask1 = getTitle();
		    }
	
		}

	   
		//SELECT  ROI
		selectROI(mask1); //custom function
	  	
	  	run("Divide...", "value=255 stack");	
		imageCalculator("Multiply create 32-bit stack", fret_nobkg,mask);
		
		//run("Bleach Correction", "correction=[Exponential Fit]");
		fret_mask = getTitle();
		imageCalculator("Multiply create 32-bit stack", donnor_nobkg,mask);
		
		//run("Bleach Correction", "correction=[Exponential Fit]");
	  	donnor_mask = getTitle();
	  	    
	    // Divide FRET by DONNOR (with no background) all image
	    imageCalculator("Divide create 32-bit stack", fret_mask, donnor_mask);
	    rename("Ratio_" + title);
	    ratio = getTitle();	  
	  
		run("Select None");
		
		//change LUT
	    run("royal");
		
	    getStatistics(area, mean, min, max, std, histogram);
	   
	    run("Enhance Contrast", "saturated=0.35");
	    wait(200);
	    print(title);
	    saveAs("Tiff", output_dir + File.separator + title + "_Ratio.tif");
	    run("RGB Color");
	    save_results(output_dir);

	    waitForUser(" ");

	    run("Close All");	    
	    close("Roi Manager");
	    if(isOpen("Results")){
       		list1 = getList("window.titles"); 
     		for (l=0; l<list1.length; l++){ 
   	    		winame = list1[l]; 
     			selectWindow(winame); 
     			run("Close"); 
     		} 	    	
	    }
}

//////NOT TESTED
//Fret ratio analysis for time-lapse
function TimeLapse_FRET_ratio(original){		
		
	    selectWindow(original);

	    //correct illumination
	    run("Polynomial Shading Corrector", "degree_x=1 degree_y=1 regularization=10");	    
	    run("Split Channels"); 
    	selectWindow("C" + donnor_channel + "-" + original);
    	donnor = getTitle();
    	selectWindow("C" + fret_channel + "-" + original);
    	fret = getTitle(); 	 
	    
	    selectWindow(fret);
	    setTool("rectangle");
	    waitForUser("Please select background with rectangular tool. Click OK when done");
 	    getBoundingRect(xmin, ymin, width, height);
 	    getSelectionCoordinates(x, y);
        makeSelection(0, x, y);
	    getStatistics(area,mean_fret);
	    print("Substracted BKG value: ",mean_fret);
	            	    
		//subtract fret background 	
		run("Select None");    
	    selectWindow(fret);
	    run("Subtract...", "value=" + mean_fret + " stack");	
	    fret_nobkg = getTitle();

		//subtract donnor background 
	    selectWindow(donnor);
    	makeSelection(0, x, y);
	    getStatistics(area,mean_donnor);
	    print("Substracted BKG value: ",mean_donnor);
	    run("Select None");   
    	run("Subtract...", "value=" + mean_donnor + " stack");
    	donnor_nobkg = getTitle();	
	   	    
	    //cell segmentation => ROI mask
    	selectWindow(fret_nobkg);
	    run("Duplicate...", "duplicate title=tomask");
	    run("Median...", "radius=2 stack");
	    if (manual_thresh == true){
	    	waitForUser("Select Image -> Adjust -> Threshold.. and select the best algorithm. Click OK after applying to all stack");
	    }
	    else {
	    	setAutoThreshold("Li dark");
			setOption("BlackBackground", true);
			run("Convert to Mask", "method=Li background=Default black"); 	
	    }

	    //////Se a celula se mexe ao longo do tempo não ira funcionar
	  	run("Analyze Particles...", "size=100-Infinity show=Masks include add stack");
		roiManager("Select", 0);
		run("Create Mask");
		mask = getTitle();
		// devide mask by 255 (0-1)
	    run("Divide...", "value=255 atack");	


	     // Multiply the result by donnor mask in order to get only information inside the cell 
	    imageCalculator("Multiply create 32-bit stack", mask, fret_nobkg);
	     fret_masked = getTitle(); 
	     // Multiply the result by donnor mask in order to get only information inside the cell 
	    imageCalculator("Multiply create 32-bit stack", mask, donnor_nobkg);
	    donnor_masked = getTitle(); 
	    // Divide FRET by DONNOR (with no background) all image
	    imageCalculator("Divide create 32-bit stack", fret_masked, donnor_masked);
	    
	    rename("Ratio_" + title);
	    ratio = getTitle();	  
	    
	   
		//select ROI adnd measure inside it
		//create mask with only one object in the field
	   
		Dialog.create("Exclude objects...");
 		Dialog.addMessage("Is there more than one cell in the image?")
 		Dialog.addRadioButtonGroup("Answer", newArray("No", "Yes"), 1, 1, "No"); 
  	    //Dialog.addChoice("Type:", newArray("No", "yes"));
  		Dialog.show();	    
	  
	    ck_value = Dialog.getRadioButton;
		if (ck_value == "yes"){
			selectWindow(fret_nobkg);
		    setAutoThreshold("Default dark");
		    getThreshold(lower, upper);
		    setThreshold(lower*0.7,upper);
			//run("Threshold...");
			setOption("BlackBackground", false);
			run("Convert to Mask");
			run("Fill Holes", "stack");			
			run("Options...", "iterations=1 count=1 pad do=Erode stack");
			run("Watershed", "stack");			
			run("Analyze Particles...", "size=50-Infinity show=Masks exclude include add stack");
			
		    mask_enlarged=getTitle();
		    
			rois = roiManager("count");
			
			for(r = 0;r<rois;r++){
				roiManager("select", r);
				run("Enlarge...", "enlarge=3 pixel"); 
				roiManager("update");
				roiManager("fill");
				
			}
		    // Multiply the result by donnor mask in order to get only information inside the cell 
		    imageCalculator("Multiply create 32-bit stack", ratio, mask_enlarged);
		    //run("Divide...", "value=255 stack");
		    run("Measure Stack...");
	 	}
	 	else {
	 		selectWindow(ratio);
	 		run("Measure Stack...");	
	 	}
	    
	   
		//change LUT
	    run("royal");
		roiManager("Select", 0);
	    getStatistics(area, mean, min, max, std, histogram);
	   
	   // setMinAndMax(mean-3*std, mean+3*std);
	    run("Enhance Contrast", "saturated=0.35");
	    wait(200);
	    saveAs("Tiff", output_dir + File.separator + title + "_Ratio.tif");
	    run("RGB Color");
	    save_results(output_dir);

	    run("Close All");	    
	    close("Roi Manager");
	    if(isOpen("Results")){
       		list1 = getList("window.titles"); 
     		for (l=0; l<list1.length; l++){ 
   	    		winame = list1[l]; 
     			selectWindow(winame); 
     			run("Close"); 
     		} 	    	
	    }
}

// output savings
function save_results(outputdir){

		roiManager("Save", output_dir + File.separator + title + "_Roi.roi");
		
	    saveAs("Results", output_dir + File.separator + title + "_Results.xls");

	    saveAs("Tiff", output_dir + File.separator + title + "_RatioRGB.tif");
	
}

function merge_excel_files(){
	setBatchMode(true);
	
	list=getFileList(output_dir);
	L=lengthOf(list);
	first=1;

	for (i=0; i<L; i++){

		if(endsWith(list[i],".xls") && (list[i] != "Final_Results.xls")){
			fname=output_dir+ File.separator + list[i];
			open(fname);
			if(first==1){				
				IJ.renameResults("Total");
				first=0; 	
			}				
			else{
				IJ.renameResults("Results");
				nr = nResults;
				label = newArray(nr); 
				area = newArray(nr); 
				mean = newArray(nr); 
				min = newArray(nr); 
				max = newArray(nr); 
				den = newArray(nr); 
				rawden = newArray(nr); 
				
				for(j=0;j<nr;j++)
				{
					//COPY
				
					label[j]=getResultString("Label",j);
					
					area[j]=getResult("Area",j);
					mean[j]=getResult("Mean",j);
					max[j]=getResult("Min",j);
					min[j]=getResult("Max",j);
					den[j]=getResult("IntDen",j);
					rawden[j]=getResult("RawIntDen",j);
				}
				selectWindow("Results");
				run("Close");
				wait(100);
				
				selectWindow("Total");
				IJ.renameResults("Results"); 
				nt=nResults;
				for(j=0;j<nr;j++)
				{
				//PASTE
					setResult("Label",nt+j,label[j]);
					setResult("Area",nt+j,area[j]);
					setResult("Mean",nt+j,mean[j]);
					setResult("Max",nt+j,max[j]);
					setResult("Min",nt+j,min[j]);
					setResult("IntDen",nt+j,den[j]);
					setResult("RawIntDen",nt+j,rawden[j]);					
				}
				selectWindow("Results");
				IJ.renameResults("Total");
			}
					
		}
	}
	//save
	selectWindow("Total");
	IJ.renameResults("Results");
	saveAs("Results", output_dir + File.separator + "Final_Results.xls");
	setBatchMode(false);

}

//Calculate the distance between each ROI 
function distance (x1, y1, x2, y2) { 
 	sum_difference_squared = pow((x2 - x1),2) + pow((y2 - y1),2); 
	output = pow(sum_difference_squared, 0.5); 
 	return output; //name of return variable and function cannot be same 
} 

function selectROI(img){
		Dialog.create("Exclude objects...");
 		Dialog.addMessage("Is there more than one cell in the image?")
 		Dialog.addRadioButtonGroup("Answer", newArray("No", "Yes"), 1, 1, "No"); 
  	    //Dialog.addChoice("Type:", newArray("No", "yes"));
  		Dialog.show();	    
	  
	    ck_value = Dialog.getRadioButton;
	    waitForUser(" ");
		if (ck_value == "yes"){
			//selectWindow(img);
			getDimensions(width, height, channels, slices, frames);
			run("Properties...", "channels="+channels+ " slices="+ slices + " frames = " + frames + " unit=pixels pixel_width=1 pixel_height=1 voxel_depth=1.0000000");
			setTool("point");
		    waitForUser("Select the ROI of interest");
		    getCursorLoc(centerx, centery, centerz, modifiers);
		    print(centerx,centery);
		    //setTool("wand");
		    for(s=1;s<=nSlices;s++){
		    	setSlice(s);
		    	doWand(centerx, centery);
		    	run("Wand Tool...", "tolerance=0 mode=Legacy");
		    	doWand(centerx, centery);
		    	waitForUser(" ");
		    	roiManager("Add");
		    	run("Invert", "slice");
		    	run("Clear Outside", "slice");
		    	roiManager("Deselect");
		    	run("Invert", "slice");
		    }
		}
		else{
			run("Analyze Particles...", "size=100-Infinity show=Masks exclude add stack");
		}
		   	
}

}