/* Normalize histogram values for a group of FRET Ratio images (Ana Seixas) 
 * v0.1
*
* Read all Tiff files from output folder of Fret_ratiometrics macro; 
* A scaling factor of 1000 was specified as a multiplication factor for all the ratio calculation. 
* The ratio values were normalized to the lower scale value (1.0). To avoid low intensity pixels, 
* bottom 5% of the total histogram distribution was excluded. Finally, ratiometeric images were subjected to a 
* Gaussian filter of width 2 pixels (0.32 μm) to reduce pixel noise.  
* 
* This macro should NOT be redistributed without author's permission. 
* Explicit acknowledgement to the ALM facility should be done in case of published articles (approved in C.E. 7/17/2017):     
* 
* "The authors acknowledge the support of i3S Scientific Platform Advanced Light Microscopy, 
* member of the national infrastructure PPBI-Portuguese Platform of BioImaging (supported by POCI-01-0145-FEDER-022122)."
* 
* Date: October/2019
* Author: Mafalda Sousa, mafsousa@ibmc.up.pt 
* Advanced Ligth Microscopy, I3S 
* PPBI-Portuguese Platform of BioImaging
*/

#@ File (label = "Input directory", style = "directory") input
#@ File (label = "Output directory", style = "directory") output
#@ Integer (label = "Histogram Threshold", value = 20) th_counts
#@ String (label = "File suffix", value = ".tif") suffix

// See also Process_Folder.py for a version of this code
// in the Python scripting language.

run("Close All");
total_histogram_values = newArray(0);
total_histogram = processFolder(input,total_histogram_values);
calibration_values = analyseResultsTable(total_histogram);
setContrastFolder(input,calibration_values);
// function to scan folders/subfolders/files to find files with correct suffix
function processFolder(input, total_histogram_values) {
	list = getFileList(input);
	list = Array.sort(list);
	for (i = 0; i < list.length; i++) {
		if(File.isDirectory(input + File.separator + list[i]))
			processFolder(input + File.separator + list[i],total_histogram_values);
		if(endsWith(list[i], suffix))
			total_histogram_values = processFile(input, list[i], total_histogram_values);
	}
	return total_histogram_values;
}

function processFile(input, file,total_histogram_values) {

	print("Processing: " + input + File.separator + file);
	open(input + File.separator + file);
	//multiply by a factor of 1000
	//run("Multiply...","value = 1000");
	roiManager("add");
	roiManager("select", 0);
	roiManager("measure");

	//get histogram value
	
  	nBins = 256;
  	getHistogram(values, counts, nBins);
  	new_counts = newArray(0);
  	new_values = newArray(0); 
  	//Remove the tail of the histogram with less then 2 counts per bin (~10%)
  	
  	for (i=0; i<nBins; i++) {
  		if (counts[i]>th_counts){
  			new_counts = append(new_counts,counts[i]);
  			new_values = append(new_values,values[i]);
  		}
  	}
  	p = new_counts.length/counts.length;
  	print("Percentage of histogram removal:",100-p*100);

	//Create an array to store the histogram values from comparing figures
	h1 = Array.concat(total_histogram_values,new_values);
	roiManager("reset");
	close();
	return h1;

}

function analyseResultsTable(hist_array){
	//sort total histogram array
	
	Array.sort(hist_array);
	//Exclude bottom 5% of the histogram
	//h2 = Array.slice(hist_array,round(hist_array.length*0.05),hist_array.length); 
	final_histogram = hist_array;
	Array.show(final_histogram);
	new_min = final_histogram[0];
	new_max = final_histogram[final_histogram.length-1];

	print("Minimum value of the histogram:", new_min);
	print("Maximum value of the histogram:", new_max);
	
	//Normalize the histogram o the lower scale value (1.0)
	calibration_values = newArray(new_min/new_min,new_max/new_min);

	return calibration_values; 
}


function setContrastFolder(input, calibration_values) {
	list = getFileList(input);
	list = Array.sort(list);
	for (i = 0; i < list.length; i++) {
		if(File.isDirectory(input + File.separator + list[i]))
			setContrastFolder(input + File.separator + list[i]);
		if(endsWith(list[i], suffix))
			setContrastFile(input, list[i],output,calibration_values);
	}
}

function setContrastFile(input, file, output, calibration_values) {

	print("Processing: " + input + File.separator + file);
	open(input + File.separator + file);

	//Set histogram of brightness and  contrast to the calibration values
	setMinAndMax(calibration_values[0],calibration_values[1]);
	
	//run("Calibration Bar...", "location=[Upper Right] fill=Black label=White number=3 decimal=0 font=12 zoom=1 overlay");
	//run("Multiply...","value = 1000");
	//Smooth
	//run("Gaussian Blur...", "sigma=0.05 scaled");
	
	saveAs("tiff", output + File.separator + file + "_calibrated_tiff.tiff");
	
	run("Calibration Bar...", "location=[Upper Right] fill=Black label=White number=3 decimal=0 font=12 zoom=1 overlay");
	run("RGB Color");

	saveAs("png", output + File.separator + file + "_calibrated_rgb.png");
	run("Close All");

}

function append(arr, value) {
     arr2 = newArray(arr.length+1);
     for (i=0; i<arr.length; i++)
        arr2[i] = arr[i];
     arr2[arr.length] = value;
     return arr2;
  }
